/************************************************************
  Lyrics displaying Plugin for xmms
                        Daisuke Taruki<taruki@ma.neweb.ne.jp>
************************************************************/


#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <sys/stat.h>
#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h>
#include "lyrics.h"

#include "intl/intl.h"

#ifdef JCONV
#include <jconv.h>
#else
#define jconv_info_maybe_init() 
#define jconv_info_set_locale()
#define convert_kanji_auto(i) (i)
#endif

#define CODECONV(i) convert_kanji_auto(i)

#define LOCALEDIR "/usr/share/locale"
#define PACKAGE "lyrics"

#ifdef DEBUG 
#define DPRINT(x) printf(x)
#else
#define DPRINT(x)
#endif

#define LYRICS_PATH_ENV "LYRICS_PATH"
#define LYRICS_PATH_ENV_LABEL "//LYRICS_PATH_ENV//"
#define CONFDIR ".xmms"
#define LYRICS_OPT_FILE ".lyrics"

#define OP_SHOW_TIME_TAG 0
#define OP_WORD_POSISION 1
#define OP_LYRICS_OFFSET 2
#define OP_ORDER_ENV_LRC 3
#define OP_ORDER_ENV_TXT 4
#define OP_ORDER_DIR_LRC 5

#define D_OP_WORD_CENTER 0
#define D_OP_WORD_LEFT 1
#define D_OP_WORD_RIGHT 2

#define D_CB_SAVE 0
#define D_CB_SHOW_TIME_TAG 1
#define D_CB_WORD 2

gint CB_SAVE = D_CB_SAVE;
gint CB_SHOW_TIME_TAG = D_CB_SHOW_TIME_TAG;
gint CB_WORD = D_CB_WORD;

#define NUM_OF_OPTIONS 7
static char lyrics_options[NUM_OF_OPTIONS];
static char lyrics_local_options[NUM_OF_OPTIONS];

#define TIME_MAX 65535

LYRICS *lyrics;
LYRICS *lyrics_current;

GtkWidget *l_win=NULL;
GtkWidget *list;
GtkWidget *save_button;
GtkWidget *edit_button;

GtkWidget *opt_win;
GtkWidget *opt_clist;

/* xmms plugin information structure */
static void init(void);
static void lyrics_cleanup(void);
static void lyrics_draw_func(void);
static void lyrics_playback_start(void);
static char current_playfilename[4096];

VisPlugin ly_vp =
{
  NULL,
  NULL,
  0,
  "Lyrics Plugin(0.5)",
  0,
  1,
  init,
  lyrics_cleanup,
  NULL,
  NULL,
  NULL,
  lyrics_playback_start,
  NULL,
  NULL,
  lyrics_draw_func
};

/* Current Displaying Lyricspoint; */
static unsigned int lyrics_status;

static char edit_flag;
static unsigned int play_time;
static char current_lyrics_filename[4096];

static int l_count;
static int menu_sel[3];

/* Scrolled Window Vadjustment */
GtkAdjustment *va;

void edit_func (GtkWidget *widget, gpointer data);
void lyrics_save_func (GtkWidget *widget, gpointer data);
void set_time_tag (GtkWidget *widget,
		   gint row,
		   gint column,
		   GdkEventButton *event,
		   gpointer data);
void load_opt(void);
void save_opt(void);
void set_opt(void);
void init_opt_win(void);
void opt_callback (GtkWidget *widget, gint *data);
void lyrics_opt_menu_callback (gint *data);
void display_opt_win ( void );
gint l_close (GtkWidget *widget, gpointer *gptr );
gint opt_close (GtkWidget *widget, gpointer *gptr );

#define LYRICS_SEARCH_NUM 4
LYRICS_SEARCH_ORDER lyrics_search_order[LYRICS_SEARCH_NUM]=
{
  {0, LYRICS_PATH_ENV_LABEL, "lrc"},
  {1, LYRICS_PATH_ENV_LABEL, "txt"},
  {2, NULL, "lrc" },
  {3, NULL, "txt"}
};

GeneralPlugin *get_vplugin_info(void)
{
        return &ly_vp;
}

int fscanline ( FILE *fp )
{
  int lines=0;
  fseek(fp,0,SEEK_SET);

  while ( !feof(fp) ){
    if ( (char)fgetc(fp)=='\n' ) lines++;
  }
  return ( lines );
}

int fgetline(char *BUFF, FILE *fp, int line)
{
  int count=0;
  int r_count=0;
  char dat;

  fseek(fp,0,SEEK_SET);
  while ( !feof(fp) ){
    if ( count==line ) {
      while(r_count<255){
	dat=(char)fgetc(fp);
	if ( dat != '\r' && dat != '\n' ) {*(BUFF+r_count)=dat; r_count++;}
	else break;
      }
      *(BUFF+r_count)='\0';
      return ( 0 );
    }
    if ( fgetc(fp)=='\n' ) {
      count++;
    }
  }
  return ( -1 );
}

char *mp3_to_txt ( char *BUFF, char *path, char *filename, char *ext )
{
  char *base_filename;
  int i,j;

  if ( filename == NULL ) return (NULL);

  j=i=strlen(filename)-1;

  if ( path!=NULL ) {
    while (*(filename+i)!='/')
      i--;
    base_filename=filename+i+1;
    if ( !strcmp(path, LYRICS_PATH_ENV_LABEL) )
      sprintf(BUFF,"%s/%s", getenv(LYRICS_PATH_ENV), base_filename);
    else
      sprintf(BUFF,"%s/%s", path, base_filename);
  }
  else
    strcpy(BUFF, filename);

  while ( *(BUFF+j)!='.' )
    j--;
  strcpy(BUFF+j+1, ext);

  DPRINT("Lyrios Filename:");
  DPRINT(BUFF);
  DPRINT("\n");
  return ( BUFF );
}

LYRICS *parce_lyrics(LYRICS *prev, char *buff )
{
  int min, sec, msec;
  LYRICS *ly;
  LYRICS *ly2;
  char flag=0;
  int offset=0;
  int last_tag=6;

  ly=(LYRICS *)malloc(sizeof(LYRICS));
  if ( prev!=NULL )
    prev->next=ly;

  ly->prev=prev;
  ly->next=NULL;

  if ( strlen(buff) < 7 ||
       *(buff+0)!='[' ||
       sscanf(buff,"[%d:%d]", &(min), &(sec) ) < 2) {
    ly->have_time=0;
    ly->time=-1;
    strcpy(ly->word, buff+offset);
  }
  else {
    if (min>99) last_tag++;
    if ( *(buff+last_tag) == ':' ) {
      if ( sscanf("",&(msec)) < 1) {
	ly->msec=msec;
      }
      else ly->msec = 0;
      offset=last_tag+4;
    }
    else {
      offset = last_tag+1;
      ly->msec=0;
    }
    ly->have_time=1;
    ly->time=min*60+sec;
    if ( *(buff+offset)!='[' ) {
      strcpy(ly->word, buff+offset);
    }
    else {
      flag=1;
      ly2=parce_lyrics(ly, buff+offset );
      strcpy(ly->word, ly2->word);
    }
  }

  if ( flag != 0 )
    return ( ly2 );
  return (ly);
}

LYRICS *ly_getwrdtxt(char *filename)
{
  FILE *fp=NULL;
  char BUFF[4096];
  char *lyrics_path;
  LYRICS *l_tmp=NULL;
  LYRICS *ltop=NULL;
  LYRICS *llast=NULL;
  LYRICS *lsort[4];
  int lines;
  int i, j;
  char flag;
  struct stat stat_buf;

  DPRINT ( "ly_getwrdtxt()\n" );
  
  lyrics_path=getenv(LYRICS_PATH_ENV);
  /* Find Lyrics file */
  for ( i = 0; i < LYRICS_SEARCH_NUM; i++ ){
    for ( j = 0; j < LYRICS_SEARCH_NUM; j++ ) {
      if ( j == lyrics_search_order[i].order ) {
	flag=stat( mp3_to_txt(BUFF, lyrics_search_order[i].path,
			      filename,
			      lyrics_search_order[i].ext),
		   &stat_buf);
	break;
      }
      if ( flag==0 )
	break;
    }
  }
  if ( flag!=0 ) return (NULL);

  fp=fopen(BUFF,"r");

  if ( fp == NULL ) {
      return ( NULL );
  }
  else
    strcpy(current_lyrics_filename,BUFF);

  lines=fscanline(fp);

  for ( i = 0; i < lines; i++) {
    if ( fgetline(BUFF,fp,i) < 0 ) {
       clr_lyrics(ltop);
       return(NULL);
    }

    if (ltop==NULL) 
      llast=ltop=parce_lyrics(NULL, BUFF );
    else 
      llast=parce_lyrics(llast, BUFF );
  }
  fclose (fp);

  l_tmp=llast;
  if ( llast->time==-1 )
    llast->time=TIME_MAX;
  while ( l_tmp!=NULL ) {
    if ( l_tmp->time==-1 )
      l_tmp->time=l_tmp->next->time;
    l_tmp=l_tmp->prev;
  }

  /* Sort Lyrics */
  
  flag=1;
  while(flag!=0) {
    flag=0;
    l_tmp=ltop;
    while(l_tmp!=NULL && flag == 0) {
      if ( l_tmp->next != NULL && (l_tmp->time) > (l_tmp->next->time) ) {
	lsort[0]=l_tmp->prev;
	lsort[1]=l_tmp;
	lsort[2]=l_tmp->next;
	lsort[3]=l_tmp->next->next;

	if (l_tmp->prev!=NULL)
	  l_tmp->prev->next=lsort[2];

	l_tmp->next->prev=lsort[0];
	l_tmp->prev=lsort[2];

	if (l_tmp->next->next!=NULL)
	  l_tmp->next->next->prev=lsort[1];
	l_tmp->next->next=lsort[1];

	l_tmp->next=lsort[3];
	flag=1;
      }
      else
	l_tmp=l_tmp->next;
    }
  }
  
#ifdef DEBUG
  l_tmp=ltop;
  while(l_tmp!=NULL){
    printf ( "[%d]%s\n", l_tmp->time, CODECONV(l_tmp->word));
    l_tmp=l_tmp->next;
  }
#endif

  return ( ltop );
}

void clr_lyrics ( LYRICS *lyrics )
{
  LYRICS *tmp;
  while ( lyrics != NULL ) {
    tmp=lyrics->next;
    free(lyrics);
    lyrics=tmp;
  }
  lyrics=lyrics_current=NULL;
}

gint l_close (GtkWidget *widget, gpointer *gptr )
{
  ly_vp.disable_plugin(&ly_vp);

  return(TRUE);
}

static void init(void)
{
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);

  l_win=init_l_win();

  gtk_widget_show(l_win); 
}

void lyrics_playback_start()
{
    open_lyrics(l_win, current_playfilename,0);
}

static void lyrics_draw_func()
{
  if (strcmp(current_playfilename, xmms_remote_get_playlist_file(ly_vp.xmms_session, 
								 xmms_remote_get_playlist_pos(ly_vp.xmms_session))) ) {
    strcpy(current_playfilename, xmms_remote_get_playlist_file(ly_vp.xmms_session, 
							       xmms_remote_get_playlist_pos(ly_vp.xmms_session)));

    open_lyrics(l_win, current_playfilename,0);
  }

  if ( l_win && play_time != xmms_remote_get_output_time(ly_vp.xmms_session)/1000 ) {
    play_time = xmms_remote_get_output_time(ly_vp.xmms_session)/1000;
    update_lyrics(play_time);
  }
}

static void lyrics_cleanup(void)
{
  if (l_count!=0){
    gtk_clist_clear(GTK_CLIST(list));
    l_count=0;
  }
  clr_lyrics(lyrics);
  if(opt_win) gtk_widget_destroy(opt_win);
  opt_win=NULL;
  if(l_win) gtk_widget_destroy(l_win);
//  gtk_widget_hide(l_win);
  l_win=NULL;
}

GtkWidget *init_l_win(void)
{
  GtkWidget *vbox, *hbox;
  GtkWidget *button;
  GtkWidget *scrolled_win;
  GtkWidget *lyrics_disp_win;
  lyrics=NULL;
  l_count=0;
  edit_flag=0;

  DPRINT ( "init_l_win();\n" );
  lyrics_disp_win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name( lyrics_disp_win, "l_win" );
  gtk_widget_set_usize (lyrics_disp_win, 400, 300);

  gtk_signal_connect (GTK_OBJECT (lyrics_disp_win), "destroy",
                      GTK_SIGNAL_FUNC (l_close), NULL);
  gtk_signal_connect_object (GTK_OBJECT (lyrics_disp_win), "delete_event",
                             GTK_SIGNAL_FUNC (l_close),
                             GTK_OBJECT (lyrics_disp_win));
  
  gtk_container_border_width (GTK_CONTAINER (lyrics_disp_win), 0);
    
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (lyrics_disp_win), vbox);
  gtk_widget_show (vbox);

  scrolled_win = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
				  GTK_POLICY_AUTOMATIC, 
				  GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox), scrolled_win, TRUE, TRUE, 0);
  va = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolled_win));
  gtk_widget_show (scrolled_win);
  list = gtk_clist_new (2);

  gtk_clist_set_column_width(GTK_CLIST(list), 0, 0);
  gtk_clist_set_column_width(GTK_CLIST(list), 1, 300);

  gtk_clist_set_selection_mode(GTK_CLIST(list), GTK_SELECTION_SINGLE);
  gtk_container_add (GTK_CONTAINER (scrolled_win), list);
  gtk_container_set_focus_vadjustment (GTK_CONTAINER (scrolled_win), va);

  gtk_widget_show (list);

  gtk_signal_connect(GTK_OBJECT(list), "select_row",
                     GTK_SIGNAL_FUNC(set_time_tag), NULL );

  hbox = gtk_hbox_new (FALSE, 10);
  gtk_container_border_width (GTK_CONTAINER (hbox), 10);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  button = gtk_button_new_with_label (_("Close"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC(l_close),
		      NULL );
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
  gtk_widget_show (button);

  button = gtk_button_new_with_label (_("Preference"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC(display_opt_win),
		      NULL );
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
  gtk_widget_show (button);

  edit_button = gtk_toggle_button_new_with_label (_("Edit"));
  gtk_signal_connect (GTK_OBJECT (edit_button), "clicked",
		      GTK_SIGNAL_FUNC(edit_func),
		      NULL );
  gtk_box_pack_start (GTK_BOX (hbox), edit_button, FALSE, TRUE, 0);

  save_button = gtk_button_new_with_label (_("Save"));
  gtk_signal_connect (GTK_OBJECT (save_button), "clicked",
		      GTK_SIGNAL_FUNC(lyrics_save_func),
		      NULL );
  gtk_box_pack_start (GTK_BOX (hbox), save_button, FALSE, TRUE, 0);

  /* Load Options */
  load_opt();
  set_opt();
  init_opt_win();

  return ( lyrics_disp_win );
}

void display_opt_win ( void )
{
  gtk_widget_show(opt_win);
}

void open_lyrics ( GtkWidget *l_win, char *filename, gchar show_flag )
{
  LYRICS *l_tmp;
  char BUFF[255];
  char *clist_item[2];

  set_opt();

  DPRINT("open lyrics file and setting()\n");
  play_time=0;
  if ( edit_flag != 0 ) return;

  lyrics_status=0;
  clr_lyrics(lyrics);
  l_tmp=lyrics=ly_getwrdtxt(filename);

  DPRINT("\tset window title()\n");
  if ( lyrics != NULL ){
    sprintf (BUFF, "Lyrics - %s -", current_lyrics_filename );
    gtk_window_set_title(GTK_WINDOW(l_win), BUFF);
    gtk_widget_show (edit_button);
  }
  else
  {
    sprintf (BUFF, "Lyrics - %s -", "No Lyrics File" );
    gtk_window_set_title(GTK_WINDOW(l_win), BUFF);
    gtk_widget_hide (edit_button);
  }

  gtk_clist_freeze(GTK_CLIST(list));
  DPRINT("\tclist clear()\n");
  if (l_count!=0);{
    gtk_clist_clear(GTK_CLIST(list));
    l_count=0;
  }
  DPRINT("\tadd items to clist ()\n");
  while (l_tmp!=NULL) {
    clist_item[0]=(char *)malloc(16);
    clist_item[1]=(char *)malloc(256);
    if ( l_tmp->have_time!=0)
      sprintf(clist_item[0]," [%02d:%02d]", l_tmp->time/60, l_tmp->time%60);
    else
      strcpy(clist_item[0],"");
    strncpy(clist_item[1], CODECONV(l_tmp->word), 256);
    gtk_clist_append(GTK_CLIST(list), clist_item);
    gtk_clist_set_row_data( GTK_CLIST(list), l_count, l_tmp );
    l_count++;
    l_tmp=l_tmp->next;
    free(clist_item[0]);
    free(clist_item[1]);
  }
  gtk_clist_thaw(GTK_CLIST(list));

  DPRINT("\tlyrics window show()\n");
  if ( show_flag == 0 ) return; 
  if ( lyrics!=NULL ) {
    gtk_widget_show(l_win); 
    }
  else
    gtk_widget_hide(l_win); 
}

void update_lyrics ( long time )
{
  LYRICS *lyrics_tmp;
  float current_view;
  
  DPRINT("update lyrics status()\n");

  if ( edit_flag != 0 ) return;

  lyrics_status=0;

  lyrics_tmp=lyrics;
  if ( lyrics_tmp==NULL || l_count==0 ) return;

  DPRINT("\ttime seek\n");
  while ( lyrics_tmp->time <= time ) {
    lyrics_status++;
    lyrics_tmp=lyrics_tmp->next;
    if ( lyrics_tmp==NULL ) break;
  }  
  if ( lyrics_status==0 || lyrics_tmp == lyrics_current ) return;

  GDK_THREADS_ENTER();

  DPRINT("\tselect item\n");
  lyrics_current=lyrics_tmp;
  gtk_clist_select_row(GTK_CLIST(list),lyrics_status-1,0 );

  /* Auto Scroll  */
  DPRINT("\tmove view adgustment\n");
  current_view=va->upper*((float)(lyrics_status)/(float)l_count);
  if ( current_view < va->page_size/2.0 )
    gtk_adjustment_set_value(va, va->lower);
  else if ( current_view > (va->upper - va->page_size/2.0) )
    gtk_adjustment_set_value(va, va->upper-(va->page_size));
  else
    gtk_adjustment_set_value(va, current_view-(va->page_size/2.0));

  GDK_THREADS_LEAVE();

#ifdef DEBUG
    printf ("va=%f ls=%d lc=%d\n",va->value,lyrics_status,l_count);
    printf ("ps=%f upper=%f lower=%f\n",va->page_size,va->upper,va->lower);
#endif

}

void edit_func (GtkWidget *widget, gpointer data)
{
  if (GTK_TOGGLE_BUTTON (widget)->active) {
    edit_flag=1;
    gtk_widget_show (save_button);
  }
  else{
    edit_flag=0;
    gtk_widget_hide (save_button);
  }
}

void set_time_tag (GtkWidget *widget,
		   gint row,
		   gint column,
		   GdkEventButton *event,
		   gpointer data)
{
  char buff[255];
  LYRICS *edit_lyrics;

  if ( edit_flag==0 ) return;

  sprintf ( buff, "*[%02d:%02d]", play_time/60, play_time%60 );
  gtk_clist_set_text( GTK_CLIST(widget), row, 0, buff );
  edit_lyrics = ((LYRICS *)gtk_clist_get_row_data( GTK_CLIST(widget), row ));
  edit_lyrics->have_time=1;
  edit_lyrics->time=play_time;

}

void lyrics_save_func (GtkWidget *widget, gpointer data)
{
  LYRICS *l_tmp;
  FILE *fp;

  l_tmp=lyrics;

  fp = fopen ( current_lyrics_filename, "w" );
  if ( fp != NULL ) {
    while ( l_tmp!=NULL ) {
      if ( l_tmp->have_time==1 )
	fprintf ( fp, "[%02d:%02d]%s\r\n", l_tmp->time/60, l_tmp->time%60,
		  l_tmp->word );
      else 
	fprintf ( fp, "%s\r\n",  l_tmp->word );
      l_tmp=l_tmp->next;
    }
    fclose(fp);
  }
  else {
    printf ( "Can't write to lyrics file: %s\n", current_lyrics_filename );
    printf ( "Output to stdout.\n" );
    while ( l_tmp!=NULL ) {
      if ( l_tmp->have_time==1 )
	printf ( "[%02d:%02d]%s\n", l_tmp->time/60, l_tmp->time%60,
		  CODECONV(l_tmp->word) );
      else 
	printf ( "%s\n",  CODECONV(l_tmp->word) );
      l_tmp=l_tmp->next;
    }
    fclose(fp);
  }
}

void save_opt(void)
{
  FILE *fp;
  char BUFF[4096];

  sprintf (BUFF,"%s/%s/%s", getenv("HOME"), CONFDIR, LYRICS_OPT_FILE );
  fp = fopen(BUFF, "w");
  if ( fp==NULL) return;
  fwrite(lyrics_options, sizeof(char), NUM_OF_OPTIONS, fp);
  fclose(fp);
}

void load_opt(void)
{
  FILE *fp;
  char BUFF[4096];
  struct stat stat_buf;
  int i;

  DPRINT("Load options();\n");
  for (i=0; i < 2; i++ ){
    lyrics_options[i]=0;
  }
  for (i=0; i < 4; i++ )
    lyrics_search_order[i].order=lyrics_options[i+2]=i;

  sprintf (BUFF,"%s/%s/%s", getenv("HOME"), CONFDIR, LYRICS_OPT_FILE );
  if (!stat(BUFF, &stat_buf)) {
    fp = fopen(BUFF, "r");
    fread(lyrics_options, sizeof(char), NUM_OF_OPTIONS, fp);
    fclose(fp);
  }
  for (i=0; i < NUM_OF_OPTIONS; i++ ){
    lyrics_local_options[i]=lyrics_options[i];
  }
}

void set_opt(void)
{
  DPRINT("Set Options\n");

  DPRINT("\tShow Time tag\n");
  if ( lyrics_options[OP_SHOW_TIME_TAG]==1 )
    gtk_clist_set_column_width(GTK_CLIST(list), 0, 64);
  else
    gtk_clist_set_column_width(GTK_CLIST(list), 0, 0);

  DPRINT("\tSet Word Justification\n");
  switch ( lyrics_options[OP_WORD_POSISION] ){
  case D_OP_WORD_CENTER:
    gtk_clist_set_column_justification( GTK_CLIST(list), 1, GTK_JUSTIFY_CENTER);
    break;
  case D_OP_WORD_LEFT:
    gtk_clist_set_column_justification( GTK_CLIST(list), 1, GTK_JUSTIFY_LEFT);
    break;
  case D_OP_WORD_RIGHT:
    gtk_clist_set_column_justification( GTK_CLIST(list), 1, GTK_JUSTIFY_RIGHT);
    break;
  }
}

void init_opt_win(void)
{
  GtkWidget *table;
  GtkWidget *button;
  GtkWidget *menu_item;
  GtkWidget *menu;
  GtkWidget *menu_button;
  char *cl_item[2];
  int i;
  char *menu_label[]={_("Center"),_("Left"),_("Right")};

  DPRINT ( "init_opt_win();\n" );

  for ( i=0; i < NUM_OF_OPTIONS; i ++ )
    lyrics_local_options[i]=lyrics_options[i];

  opt_win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name( opt_win, "opt_win" );
  gtk_widget_set_usize (opt_win, 270, 200);
  gtk_window_set_title(GTK_WINDOW(opt_win), _("Lyrics Window Preference"));

  gtk_signal_connect (GTK_OBJECT (opt_win), "destroy",
                     GTK_SIGNAL_FUNC (opt_close), NULL );

  gtk_signal_connect_object (GTK_OBJECT (opt_win), "delete_event",
                             GTK_SIGNAL_FUNC (opt_close),
                             GTK_OBJECT (opt_win));

  gtk_container_border_width (GTK_CONTAINER (opt_win), 0);

  table = gtk_table_new(7, 7, FALSE );
  gtk_table_set_row_spacings(GTK_TABLE(table), 10);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
  gtk_container_add(GTK_CONTAINER(opt_win), table);
  gtk_widget_show(table);

  opt_clist=gtk_clist_new(2);
  gtk_clist_set_column_width(GTK_CLIST(opt_clist), 0, 64);
  gtk_clist_set_column_width(GTK_CLIST(opt_clist), 1, 100);
  gtk_table_attach_defaults(GTK_TABLE(table),opt_clist,3,5,3,4);
  cl_item[0]=(char *)malloc(16);
  cl_item[1]=(char *)malloc(50);
  if ( lyrics_options[OP_SHOW_TIME_TAG]==1 )
    sprintf(cl_item[0], " [MM:SS]" );
  else
    sprintf(cl_item[0], " " );
  sprintf(cl_item[1], _("Lyrics preview") );
  gtk_clist_append(GTK_CLIST(opt_clist), cl_item);
  free(cl_item[0]);
  free(cl_item[1]);

  gtk_widget_show (opt_clist);

  button = gtk_toggle_button_new_with_label (_("Show Time TAG"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC(opt_callback),
		      (gpointer)& CB_SHOW_TIME_TAG );
  gtk_table_attach_defaults(GTK_TABLE(table),button,3,4,5,6);
  if ( lyrics_options[OP_SHOW_TIME_TAG]==1 )
    gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON(button), TRUE );
  gtk_widget_show (button);

  button=gtk_button_new_with_label(_("OK"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC(opt_callback),
		      (gpointer)& CB_SAVE  );
  gtk_table_attach_defaults(GTK_TABLE(table),button,3,4,6,7);
  gtk_widget_show (button);

  button=gtk_button_new_with_label(_("Cancel"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC(opt_close),
		      NULL );
  gtk_table_attach_defaults(GTK_TABLE(table),button,4,5,6,7);
  gtk_widget_show (button);

  menu=gtk_menu_new();

  for ( i = 0; i < 3; i ++ ) {
    menu_sel[i]=i;
    menu_item=gtk_menu_item_new_with_label(menu_label[i]);
    gtk_menu_append( GTK_MENU(menu), menu_item );
    gtk_widget_show(menu_item);
    gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			      GTK_SIGNAL_FUNC(lyrics_opt_menu_callback),
			      (gpointer)&menu_sel[i]);
  }

  menu_button = gtk_option_menu_new();
  gtk_menu_set_active(GTK_MENU(menu), lyrics_options[OP_WORD_POSISION]);
  gtk_option_menu_set_menu(GTK_OPTION_MENU(menu_button), menu);

  gtk_table_attach_defaults(GTK_TABLE(table),menu_button,4,5,5,6);
  gtk_widget_show(menu_button);

}

void opt_callback (GtkWidget *widget, gint *data)
{
  int i;

  switch(*data) {

  case D_CB_SHOW_TIME_TAG:
    if (GTK_TOGGLE_BUTTON (widget)->active) {
      lyrics_local_options[OP_SHOW_TIME_TAG]=1;
      gtk_clist_set_text(GTK_CLIST(opt_clist),0,0," [MM:SS]");
    }
    else{
      lyrics_local_options[OP_SHOW_TIME_TAG]=0;
      gtk_clist_set_text(GTK_CLIST(opt_clist),0,0," ");
    }
    break;

  case D_CB_SAVE:
    for ( i=0; i < NUM_OF_OPTIONS; i ++ )
      lyrics_options[i]=lyrics_local_options[i];
    save_opt();
    set_opt();
    opt_close ( NULL, NULL );
    break;
  }
}

void lyrics_opt_menu_callback (gint *data)
{
  switch(*data) {
  case D_OP_WORD_CENTER:
    gtk_clist_set_column_justification( GTK_CLIST(opt_clist), 1, GTK_JUSTIFY_CENTER);
    break;
  case D_OP_WORD_LEFT:
    gtk_clist_set_column_justification( GTK_CLIST(opt_clist), 1, GTK_JUSTIFY_LEFT);
    break;
  case D_OP_WORD_RIGHT:
    gtk_clist_set_column_justification( GTK_CLIST(opt_clist), 1, GTK_JUSTIFY_RIGHT);
    break;
  }
  lyrics_local_options[OP_WORD_POSISION]=*data;
}

gint opt_close (GtkWidget *widget, gpointer *gptr )
{
  gtk_widget_hide(opt_win);

  return(TRUE);
}

void show_lyrics(void)
{
  gtk_widget_show(l_win);
}
